@@ -1,6 +1,6 @@
 package App::Info::Util;
 
-# $Id: Util.pm 2272 2005-12-05 22:44:19Z theory $
+# $Id: Util.pm 2573 2006-02-04 03:53:39Z theory $
 
 =head1 NAME
 
@@ -49,7 +49,7 @@ use File::Spec ();
 use Config;
 use vars qw(@ISA $VERSION);
 @ISA = qw(File::Spec);
-$VERSION = '0.47';
+$VERSION = '0.48';
 
 my %path_dems = (MacOS   => qr',',
                  MSWin32 => qr';',
@@ -74,6 +74,8 @@ to. The constructor here is provided merely as a convenience.
 
 sub new { bless {}, ref $_[0] || $_[0] }
 
+##############################################################################
+
 =head1 OBJECT METHODS
 
 In addition to all of the methods offered by its super class,
@@ -96,6 +98,8 @@ sub first_dir {
     return;
 }
 
+##############################################################################
+
 =head2 first_path
 
   my $path = $ENV{PATH};
@@ -135,6 +139,8 @@ sub first_path {
     shift->first_dir(split /$path_dem/, shift)
 }
 
+##############################################################################
+
 =head2 first_file
 
   my $file = $util->first_file(@filelist);
@@ -151,6 +157,8 @@ sub first_file {
     return;
 }
 
+##############################################################################
+
 =head2 first_exe
 
   my $exe = $util->first_exe(@exelist);
@@ -166,6 +174,8 @@ sub first_exe {
     return;
 }
 
+##############################################################################
+
 =head2 first_cat_path
 
   my $file = $util->first_cat_path('ick.txt', @paths);
@@ -222,6 +232,8 @@ sub first_cat_path {
     return;
 }
 
+##############################################################################
+
 =head2 first_cat_dir
 
   my $dir = $util->first_cat_dir('ick.txt', @paths);
@@ -247,6 +259,8 @@ sub first_cat_dir {
     return;
 }
 
+##############################################################################
+
 =head2 first_cat_exe
 
   my $exe = $util->first_cat_exe('ick.exe', @paths);
@@ -270,6 +284,8 @@ sub first_cat_exe {
     return;
 }
 
+##############################################################################
+
 =head2 search_file
 
   my $file = 'foo.txt';
@@ -332,6 +348,37 @@ sub search_file {
     return wantarray ? @ret : $#ret <= 0 ? $ret[0] : \@ret;
 }
 
+##############################################################################
+
+=head2 files_in_dir
+
+  my @files = $util->files_in_dir($dir);
+     @files = $util->files_in_dir($dir, $filter);
+  my $files = $util->files_in_dir($dir);
+     $files = $util->files_in_dir($dir, $filter);
+
+Returns an list or array reference of all of the files and directories in the
+file system directory C<$dir>. An optional second argument is a code reference
+that filters the files. The code reference should examine the C<$_> for a file
+name and return true if it's a file that you're interested and false if it's
+not.
+
+=cut
+
+sub files_in_dir {
+    my ($self, $dir, $code) = @_;
+    return unless $dir;
+    local *DIR;
+    opendir DIR, $dir or Carp::croak "Cannot open $dir: $!\n";
+    my @files = $code
+        ? grep { $code->() } readdir DIR
+        : readdir DIR;
+    closedir DIR;
+    return wantarray ? @files : \@files;
+}
+
+##############################################################################
+
 =head2 multi_search_file
 
   my @regexen = (qr/(one)/, qr/(two)\s+(three)/);
@@ -429,6 +476,8 @@ sub multi_search_file {
     return wantarray ? @ret{@regexen} : \@ret{@regexen};
 }
 
+##############################################################################
+
 =head2 lib_dirs
 
   my @dirs = $util->lib_dirs;
@@ -471,7 +520,7 @@ L<App::Info::RDBMS::PostgreSQL|App::Info::RDBMS::PostgreSQL>
 
 Copyright (c) 2002-2004, David Wheeler. All Rights Reserved.
 
-This module is free software; you can redistribute it and/or modify it under the
-same terms as Perl itself.
+This module is free software; you can redistribute it and/or modify it under
+the same terms as Perl itself.
 
 =cut
